"""
mbed SDK
Copyright (c) 2011-2015 ARM Limited

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import re
import os
import sys
import string

from .lstools_base import MbedLsToolsBase

import logging
logger = logging.getLogger("mbedls.lstools_win7")
DEBUG = logging.DEBUG
del logging

if sys.version_info[0] < 3:
    import _winreg as winreg
else:
    import winreg


class MbedLsToolsWin7(MbedLsToolsBase):
    """ mbed-enabled platform detection for Windows
    """
    def __init__(self, **kwargs):
        MbedLsToolsBase.__init__(self, **kwargs)
        self.os_supported.append('Windows7')

    def find_candidates(self):
        return [
            {
                'mount_point': mnt,
                'target_id_usb_id': id,
                'serial_port': self._com_port(id)
            }
            for mnt, id in self.get_mbeds()
        ]

    def _com_port(self, tid):
        """! Function checks mbed serial port in Windows registry entries
        @param tid TargetID
        @return Returns None if port is not found. In normal circumstances it should never return None
        @details This goes through a whole new loop, but this assures that even if serial port (COM)
                 is not detected, we still get the rest of info like mount point etc.
        """
        winreg.Enum = winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE,
                                     r'SYSTEM\CurrentControlSet\Enum')
        usb_devs = winreg.OpenKey(winreg.Enum, 'USB')
        logger.debug("_com_port usb_devs %r",
                     list(self.iter_keys_as_str(usb_devs)))

        logger.debug("_com_port looking up usb id in all usb devices")
        dev_keys = []
        for name, vid in zip(self.iter_keys_as_str(usb_devs),
                             self.iter_keys(usb_devs)):
            try:
                dev_keys.append(winreg.OpenKey(vid, tid))
                if logger.isEnabledFor(DEBUG):
                    logger.debug(
                        "Found usb id %s in %s with subkeys %r", tid, name,
                        list(self.iter_keys_as_str(winreg.OpenKey(vid, tid))))
            except OSError:
                pass

        logger.debug("_com_port Detecting port with Device Parameter")
        for key in dev_keys:
            try:
                param = winreg.OpenKey(key, "Device Parameters")
                port, regtype = winreg.QueryValueEx(param, 'PortName')
                logger.debug('_com_port port %r regtype %r', port, regtype)
                return port
            except OSError as e:
                logger.debug("Exception %r %s", key, str(e))
                pass

        logger.debug("_com_port Detecting port by following symlinks")
        for key in dev_keys:
            try:
                ports = []
                parent_id, regtype = winreg.QueryValueEx(key, 'ParentIdPrefix')
                logger.debug("_com_port parent_id %r regtype %r")
                for VID in self.iter_keys(usb_devs):
                    candidate_keys = [k for k in self.iter_keys_as_str(VID)
                                      if parent_id in k]
                    for dev in candidate_keys:
                        logger.debug(
                            "_com_port recursive port detection with %r", dev)
                        maybe_port = self._com_port(dev)
                        if maybe_port:
                            return maybe_port
            except OSError as e:
                logger.debug("Exception %r %s", key, str(e))
                pass

        # If everything fails, return None
        return None

    def _drive_letters(self):
        """! Function for finding all drive letters
        @return Returns an array of all drive letters present
        """
        cmd = ['fsutil', 'fsinfo', 'drives']
        logger.debug('running command: %s' % (' '.join(cmd)))
        stdout, _, retval = self._run_cli_process(cmd)
        logger.debug('return code: %d' % retval)
        logger.debug('output: %s' % stdout)

        if not retval:
            drive_list = re.match('\s*Drives: (.*)', stdout.decode('utf-8')).group(1)
            return drive_list.strip().replace('\\', '').split(' ')
        return []


    def get_mbeds(self):
        """! Function filters devices' mount points for valid TargetID
        @return Returns [(<mbed_mount_point>, <mbed_id>), ..]
        @details TargetID should be a hex string with 10-48 chars
        """
        mbeds = []
        drive_letters = set(self._drive_letters())
        logger.debug('Found drive letters: %s' % ', '.join(drive_letters))
        for mbed in self.get_mbed_devices():
            mountpoint = re.match('.*\\\\(.:)$', mbed[0]).group(1)
            if mountpoint in drive_letters:
                # TargetID is a hex string with 10-48 chars
                m = re.search('[&#]([0-9A-Za-z]{10,48})[&#]', mbed[1])
                if not m:
                    continue
                tid = m.group(1)
                mbeds += [(mountpoint, tid)]
                logger.debug("get_mbeds mount_point %s usb_id %s", mountpoint, tid)
        return mbeds

    # =============================== Registry ====================================

    def iter_keys_as_str(self, key):
        """! Iterate over subkeys of a key returning subkey as string
        """
        for i in range(winreg.QueryInfoKey(key)[0]):
            yield winreg.EnumKey(key, i)

    def iter_keys(self, key):
        """! Iterate over subkeys of a key
        """
        for i in range(winreg.QueryInfoKey(key)[0]):
            yield winreg.OpenKey(key, winreg.EnumKey(key, i))

    def iter_vals(self, key):
        """! Iterate over values of a key
        """
        logger.debug("iter_vals %r", key)
        for i in range(winreg.QueryInfoKey(key)[1]):
            yield winreg.EnumValue(key, i)

    def get_mbed_devices(self):
        """! Get MBED devices (connected or not)
        @return List of devices
        @details Note: We will detect also non-standard MBED devices mentioned on 'usb_vendor_list' list.
                 This will help to detect boards like EFM boards.
        """
        upper_ven = [ven.upper() for ven in self.usb_vendor_list]
        mounts_key = winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, 'SYSTEM\MountedDevices')
        for point, label, _ in self.iter_vals(mounts_key):
            printable_label = label.decode('utf-16le', 'ignore')
            if (b'DosDevices' in point and
                any(v in printable_label.upper() for v in upper_ven)):
                logger.debug("Found Mount point %s with usb ID %s",point,
                             printable_label)
                yield (point.decode('utf-8', 'ignore'), printable_label)
            else:
                logger.debug("Skipping Mount point %r label %r", point, label)



"""
mbed SDK
Copyright (c) 2011-2015 ARM Limited

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import re
import os
import sys
import functools
from os.path import expanduser
from io import open
import json
from os import listdir
from os.path import isfile, join, exists, isdir
import logging
from abc import ABCMeta, abstractmethod

from .platform_database import PlatformDatabase, LOCAL_PLATFORM_DATABASE, \
    LOCAL_MOCKS_DATABASE
mbedls_root_logger = logging.getLogger("mbedls")
mbedls_root_logger.setLevel(logging.WARNING)

logger = logging.getLogger("mbedls.lstools_base")

def deprecated(reason):
    """Deprecate a function/method with a decorator"""
    def actual_decorator(func):
        @functools.wraps(func)
        def new_func(*args, **kwargs):
            logger.warning("Call to deprecated function %s. %s",
                        func.__name__, reason)
            return func(*args, **kwargs)
        return new_func
    return actual_decorator

class FSInteraction(object):
    BeforeFilter = 1
    AfterFilter = 2
    Never = 3

class MbedLsToolsBase(object):
    """ Base class for mbed-lstools, defines mbed-ls tools interface for
    mbed-enabled devices detection for various hosts
    """

    __metaclass__ = ABCMeta

    # Which OSs are supported by this module
    # Note: more than one OS can be supported by mbed-lstools_* module
    os_supported = []

    # Directory where we will store global (OS user specific mocking)
    HOME_DIR = expanduser("~")
    MOCK_FILE_NAME = '.mbedls-mock'
    RETARGET_FILE_NAME = 'mbedls.json'
    DETAILS_TXT_NAME = 'DETAILS.TXT'
    MBED_HTM_NAME = 'mbed.htm'

    def __init__(self, list_unmounted=False, **kwargs):
        """ ctor
        """
        self.retarget_data = {}          # Used to retarget mbed-enabled platform properties

        platform_dbs = []
        if isfile(self.MOCK_FILE_NAME) or ("force_mock" in kwargs and kwargs['force_mock']):
            platform_dbs.append(self.MOCK_FILE_NAME)
        elif isfile(LOCAL_MOCKS_DATABASE):
            platform_dbs.append(LOCAL_MOCKS_DATABASE)
        platform_dbs.append(LOCAL_PLATFORM_DATABASE)
        self.plat_db = PlatformDatabase(platform_dbs,
                                        primary_database=platform_dbs[0])
        self.list_unmounted = list_unmounted

        if 'skip_retarget' not in kwargs or not kwargs['skip_retarget']:
            self.retarget()

    @abstractmethod
    def find_candidates(self):
        """Find all candidate devices connected to this computer

        Note: Should not open any files

        @return A dict with the keys 'mount_point', 'serial_port' and 'usb_target_id'
        """
        raise NotImplemented

    @deprecated("Functionality has been moved into 'list_mbeds'. "
                "Please use list_mbeds with 'unique_names=True' and "
                "'read_details_txt=True'")
    def list_mbeds_ext(self):
        """! Function adds extra information for each mbed device
        @return Returns list of mbed devices plus extended data like 'platform_name_unique'
        @details Get information about mbeds with extended parameters/info included
        """

        return self.list_mbeds(unique_names=True, read_details_txt=True)

    def list_mbeds(
            self, fs_interaction=FSInteraction.BeforeFilter,
            platform_name_filters=[], unique_names=False,
            read_details_txt=False):
        """ List details of connected devices
        @return Returns list of structures with detailed info about each mbed
        @param fs_interaction A member of the FSInteraction class that picks the
          trade of between quality of service and speed
        @param platform_name_filters A series of regular expressions to filter
          filter targets by
        @param unique_names A boolean controlling the presence of the
          'platform_unique_name' member of the output dict
        @param read_details_text A boolean controlling the presense of the
          output dict attributes read from "DETAILS.TXT"
        @details Function returns list of dictionaries with mbed attributes 'mount_point', TargetID name etc.
        Function returns mbed list with platform names if possible
        """
        platform_count = {}
        candidates = list(self.find_candidates())
        logger.debug("Candidates for display %r", candidates)
        result = []
        platform_name_matcher = re.compile("|".join("({})".format(pf) for pf
                                                    in platform_name_filters))
        for device in candidates:
            if  ((not device['mount_point'] or
                  not self.mount_point_ready(device['mount_point'])) and
                 not self.list_unmounted):
                if  (device['target_id_usb_id'] and device['serial_port']):
                    logger.warning(
                        "MBED with target id '%s' is connected, but not mounted. "
                        "Use the '-u' flag to include it in the list.",
                        device['target_id_usb_id'])
            else:
                maybe_device = {
                    FSInteraction.BeforeFilter: self._fs_before_id_check,
                    FSInteraction.AfterFilter: self._fs_after_id_check,
                    FSInteraction.Never: self._fs_never
                }[fs_interaction](device, platform_name_matcher)
                if maybe_device:
                    if unique_names:
                        name = device['platform_name']
                        platform_count.setdefault(name, -1)
                        platform_count[name] += 1
                        device['platform_name_unique'] = (
                            "%s[%d]" % (name, platform_count[name]))
                    if read_details_txt:
                        details_txt = self._details_txt(device['mount_point']) or {}
                        device.update({"daplink_%s" % f.lower().replace(' ', '_'): v
                                       for f, v in details_txt.items()})
                    try:
                        device.update(self.retarget_data[device['target_id']])
                        logger.debug("retargeting %s to %s",
                                     device['target_id'], mbeds[i])
                    except KeyError:
                        pass
                    result.append(maybe_device)

        return result

    def _fs_never(self, device, pn):
        """Filter device without touching the file system of the device"""
        device['target_id'] = device['target_id_usb_id']
        device['target_id_mbed_htm'] = None
        device['platform_name'] = self.plat_db.get(device['target_id'][0:4])
        if device['platform_name'] and not pn.match(device['platform_name']):
            return None
        else:
            return device

    def _fs_before_id_check(self, device, pn):
        """Filter device after touching the file system of the device.
        Said another way: Touch the file system before filtering
        """
        self._update_device_from_htm(device)
        if device['platform_name'] and not pn.match(device['platform_name']):
            return None
        else:
            return device

    def _fs_after_id_check(self, device, pn):
        """Filter device before touching the file system of the device.
        Said another way: Touch the file system after filtering
        """
        plat_name = self.plat_db.get(device['target_id_usb_id'][0:4])
        if plat_name and not pn.match(plat_name):
            return None
        else:
            self._update_device_from_htm(device)
            return device

    def _update_device_from_htm(self, device):
        """Set the 'target_id', 'target_id_mbed_htm', 'platform_name' and
        'daplink_*' attributes by reading from mbed.htm on the device
        """
        htm_target_id, daplink_info = self._read_htm_ids(device['mount_point'])
        if daplink_info:
            device.update({"daplink_%s" % f.lower().replace(' ', '_'): v
                           for f, v in daplink_info.items()})
        if htm_target_id:
            logging.debug("Found htm target id, %s, for usb target id %s",
                            htm_target_id, device['target_id_usb_id'])
            device['target_id'] = htm_target_id
        else:
            logging.warning("Could not read htm on from usb id %s. "
                            "Falling back to usb id",
                            device['target_id_usb_id'])
            device['target_id'] = device['target_id_usb_id']
        device['target_id_mbed_htm'] = htm_target_id
        device['platform_name'] = self.plat_db.get(device['target_id'][0:4])

    def mock_manufacture_id(self, mid, platform_name, oper='+'):
        """! Replace (or add if manufacture id doesn't exist) entry in self.manufacture_ids
        @param oper '+' add new mock / override existing entry
                    '-' remove mid from mocking entry
        @return Mocked structure (json format)
        """
        if oper is '+':
            self.plat_db.add(mid, platform_name, permanent=True)
        elif oper is '-':
            self.plat_db.remove(mid, permanent=True)
        else:
            raise ValueError("oper can only be [+-]")

    @deprecated("List formatting methods are deprecated for a simpler API. "
                "Please use 'list_mbeds' instead.")
    def list_manufacture_ids(self):
        """! Creates list of all available mappings for target_id -> Platform
        @return String with table formatted output
        """
        from prettytable import PrettyTable

        columns = ['target_id_prefix', 'platform_name']
        pt = PrettyTable(columns)
        for col in columns:
            pt.align[col] = 'l'

        for target_id_prefix, platform_name in sorted(self.plat_db.items()):
            pt.add_row([target_id_prefix, platform_name])

        return pt.get_string()

    def retarget_read(self):
        """! Load retarget data from local file
        @return Curent retarget configuration (dictionary)
        """
        if os.path.isfile(self.RETARGET_FILE_NAME):
            logger.debug("reading retarget file %s", self.RETARGET_FILE_NAME)
            try:
                with open(self.RETARGET_FILE_NAME, "r", encoding="utf-8") as f:
                    return json.load(f)
            except IOError as e:
                logger.exception(e)
            except ValueError as e:
                logger.exception(e)
        return {}

    def retarget(self):
        """! Enable retargeting
        @details Read data from local retarget configuration file
        @return Retarget data structure read from configuration file
        """
        self.retarget_data = self.retarget_read()
        return self.retarget_data

    # Note: 'Ven_SEGGER' - This is used to detect devices from EFM family, they use Segger J-LInk to wrap MSD and CDC
    usb_vendor_list = ['Ven_MBED', 'Ven_SEGGER', 'Ven_ARM_V2M']


    def get_dummy_platform(self, platform_name):
        """! Returns simple dummy platform """
        if not hasattr(self, "dummy_counter"):
            self.dummy_counter = {} # platform<str>: counter<int>

        if platform_name not in self.dummy_counter:
            self.dummy_counter[platform_name] = 0

        platform = {
            "platform_name": platform_name,
            "platform_name_unique": "%s[%d]"% (platform_name, self.dummy_counter[platform_name]),
            "mount_point": "DUMMY",
            "serial_port": "DUMMY",
            "target_id": "DUMMY",
            "target_id_mbed_htm": "DUMMY",
            "target_id_usb_id": "DUMMY",
            "daplink_version": "DUMMY"
        }
        self.dummy_counter[platform_name] += 1
        return platform

    @deprecated("List formatting methods are deprecated to simplify the API. "
                "Please use 'list_mbeds' instead.")
    def list_platforms(self):
        """! Useful if you just want to know which platforms are currently available on the system
        @return List of (unique values) available platforms
        """
        result = []
        mbeds = self.list_mbeds()
        for i, val in enumerate(mbeds):
            platform_name = str(val['platform_name'])
            if platform_name not in result:
                result.append(platform_name)
        return result

    @deprecated("List formatting methods are deprecated to simplify the API. "
                "Please use 'list_mbeds' instead.")
    def list_platforms_ext(self):
        """! Useful if you just want to know how many platforms of each type are currently available on the system
        @return Dict of platform: platform_count
        """
        result = {}
        mbeds = self.list_mbeds()
        for i, val in enumerate(mbeds):
            platform_name = str(val['platform_name'])
            if platform_name not in result:
                result[platform_name] = 1
            else:
                result[platform_name] += 1
        return result

    @deprecated("List formatting methods are deprecated to simplify the API. "
                "Please use 'list_mbeds' instead.")
    def list_mbeds_by_targetid(self):
        """! Get information about mbeds with extended parameters/info included
        @return Returns dictionary where keys are TargetIDs and values are mbed structures
        @details Ordered by target id (key: target_id).
        """
        result = {}
        mbed_list = self.list_mbeds_ext()
        for mbed in mbed_list:
            target_id = mbed['target_id']
            result[target_id] = mbed
        return result

    def __str__(self):
        """! Object to string casting

        @return Stringified class object should be prettytable formated string
        """
        return self.get_string()

    @deprecated("List formatting methods are deprecated to simplify the API. "
                "Please use 'list_mbeds' instead.")
    def get_string(self, border=False, header=True, padding_width=1, sortby='platform_name'):
        """! Printing with some sql table like decorators
        @param border Table border visibility
        @param header Table header visibility
        @param padding_width Table padding
        @param sortby Column used to sort results
        @return Returns string which can be printed on console
        """
        from prettytable import PrettyTable
        result = ''
        mbeds = self.list_mbeds(unique_names=True, read_details_txt=True)
        if mbeds:
            """ ['platform_name', 'mount_point', 'serial_port', 'target_id'] - columns generated from USB auto-detection
                ['platform_name_unique', ...] - columns generated outside detection subsystem (OS dependent detection)
            """
            columns = ['platform_name', 'platform_name_unique', 'mount_point', 'serial_port', 'target_id', 'daplink_version']
            pt = PrettyTable(columns)
            for col in columns:
                pt.align[col] = 'l'

            for mbed in mbeds:
                row = []
                for col in columns:
                    row.append(mbed[col] if col in mbed and mbed[col] else 'unknown')
                pt.add_row(row)
            result = pt.get_string(border=border, header=header, padding_width=padding_width, sortby=sortby)
        return result

    # Private functions supporting API

    @deprecated("This method will be removed from the public API. "
                "Please use 'list_mbeds' instead")
    def get_json_data_from_file(self, json_spec_filename, verbose=False):
        """! Loads from file JSON formatted string to data structure
        @return None if JSON can be loaded
        """
        try:
            with open(json_spec_filename) as data_file:
                try:
                    return json.load(data_file)
                except ValueError as json_error_msg:
                    logging.error("Parsing file(%s): %s", json_spec_filename, json_error_msg)
                    return None
        except IOError as fileopen_error_msg:
            logging.warning(fileopen_error_msg)
            return None

    @deprecated("This method will be removed from the public API. "
                "Please use 'list_mbeds' instead")
    def get_htm_target_id(self, mount_point):
        target_id, _ = self._read_htm_ids(mount_point)
        return target_id

    @deprecated("This method will be removed from the public API. "
                "Please use 'list_mbeds' instead")
    def get_mbed_htm(self, mount_point):
        _, build_info = self._read_htm_ids(mount_point)
        return build_info

    def _read_htm_ids(self, mount_point):
        """! Function scans mbed.htm to get information about TargetID.
        @param mount_point mbed mount point (disk / drive letter)
        @return Function returns targetID, in case of failure returns None.
        @details Note: This function should be improved to scan variety of boards' mbed.htm files
        """
        result = {}
        target_id = None
        for line in self._htm_lines(mount_point):
            target_id = target_id or self._target_id_from_htm(line)
            ver_bld = self._mbed_htm_comment_section_ver_build(line)
            if ver_bld:
                result['version'], result['build'] = ver_bld

            m = re.search(r'url=([\w\d\:/\\\?\.=-_]+)', line)
            if m:
                result['url'] = m.group(1).strip()
        return target_id, result

    @deprecated("This method will be removed from the public API. "
                "Please use 'list_mbeds' instead")
    def get_mbed_htm_comment_section_ver_build(self, line):
        return self._mbed_htm_comment_section_ver_build(line)

    def _mbed_htm_comment_section_ver_build(self, line):
        """! Check for Version and Build date of interface chip firmware im mbed.htm file
        @return (version, build) tuple if successful, None if no info found
        """
        # <!-- Version: 0200 Build: Mar 26 2014 13:22:20 -->
        m = re.search(r'^<!-- Version: (\d+) Build: ([\d\w: ]+) -->', line)
        if m:
            version_str, build_str = m.groups()
            return (version_str.strip(), build_str.strip())

        # <!-- Version: 0219 Build: Feb  2 2016 15:20:54 Git Commit SHA: 0853ba0cdeae2436c52efcba0ba76a6434c200ff Git local mods:No-->
        m = re.search(r'^<!-- Version: (\d+) Build: ([\d\w: ]+) Git Commit SHA', line)
        if m:
            version_str, build_str = m.groups()
            return (version_str.strip(), build_str.strip())

        # <!-- Version: 0.14.3. build 471 -->
        m = re.search(r'^<!-- Version: ([\d+\.]+)\. build (\d+) -->', line)
        if m:
            version_str, build_str = m.groups()
            return (version_str.strip(), build_str.strip())
        return None

    @deprecated("This method will be removed from the public API. "
                "Please use 'list_mbeds' instead")
    def get_mbed_htm_lines(self, mount_point):
        return self._htm_lines(mount_point)

    def _htm_lines(self, mount_point):
        if mount_point:
            mbed_htm_path = join(mount_point, self.MBED_HTM_NAME)
            try:
                with open(mbed_htm_path, 'r') as f:
                    return f.readlines()
            except IOError:
                logger.debug('Failed to open file %s', mbed_htm_path)
        return []

    @deprecated("This method will be removed from the public API. "
                "Please use 'list_mbeds' instead")
    def get_details_txt(self, mount_point):
        return self._details_txt(mount_point)

    def _details_txt(self, mount_point):
        """! Load DETAILS.TXT to dictionary:
            DETAILS.TXT example:
            Version: 0226
            Build:   Aug 24 2015 17:06:30
            Git Commit SHA: 27a236b9fe39c674a703c5c89655fbd26b8e27e1
            Git Local mods: Yes

            or:

            # DAPLink Firmware - see https://mbed.com/daplink
            Unique ID: 0240000029164e45002f0012706e0006f301000097969900
            HIF ID: 97969900
            Auto Reset: 0
            Automation allowed: 0
            Daplink Mode: Interface
            Interface Version: 0240
            Git SHA: c765cbb590f57598756683254ca38b211693ae5e
            Local Mods: 0
            USB Interfaces: MSD, CDC, HID
            Interface CRC: 0x26764ebf
        """
        if mount_point:
            path_to_details_txt = os.path.join(mount_point, self.DETAILS_TXT_NAME)
            try:
                with open(path_to_details_txt, 'r') as f:
                    return self._parse_details(f.readlines())
            except IOError as e:
                logger.debug('Failed to open file %s: %s', path_to_details_txt, str(e))
        return None

    @deprecated("This method will be removed from the public API. "
                "Please use 'list_mbeds' instead")
    def parse_details_txt(self, lines):
        return self._parse_details(lines)

    def _parse_details(self, lines):
        result = {}
        for line in lines:
            if not line.startswith('#'):
                key, _, value = line.partition(':')
                if value:
                    result[key] = value.strip()
        if 'Interface Version' in result:
            result['Version'] = result['Interface Version']
        return result

    @deprecated("This method will be removed from the public API. "
                "Please use 'list_mbeds' instead")
    def scan_html_line_for_target_id(self, line):
        return self._target_id_from_htm(line)

    def _target_id_from_htm(self, line):
        """! Extract Target id from htm line.
        @return Target id or None
        """
        # Detecting modern mbed.htm file format
        m = re.search('\?code=([a-fA-F0-9]+)', line)
        if m:
            result = m.groups()[0]
            logger.debug("Found target id %s in htm line %s", result, line)
            return result
        # Last resort, we can try to see if old mbed.htm format is there
        m = re.search('\?auth=([a-fA-F0-9]+)', line)
        if m:
            result = m.groups()[0]
            logger.debug("Found target id %s in htm line %s", result, line)
            return result

        return None

    def mount_point_ready(self, path):
        """! Check if a mount point is ready for file operations
        """
        return exists(path) and isdir(path)

    @staticmethod
    @deprecated("This method will be removed from the public API. "
                "Please use 'list_mbeds' instead")
    def run_cli_process(cmd, shell=True):
        return MbedLsToolsBase._run_cli_process(cmd, shell)

    @staticmethod
    def _run_cli_process(cmd, shell=True):
        """! Runs command as a process and return stdout, stderr and ret code
        @param cmd Command to execute
        @return Tuple of (stdout, stderr, returncode)
        """
        from subprocess import Popen, PIPE

        p = Popen(cmd, shell=shell, stdout=PIPE, stderr=PIPE)
        _stdout, _stderr = p.communicate()
        return _stdout, _stderr, p.returncode



#!/usr/bin/env python
"""
mbed SDK
Copyright (c) 2011-2015 ARM Limited

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import unittest
import os
import errno
import logging

from mbed_lstools.main import create



class ParseMbedHTMTestCase(unittest.TestCase):
    """ Unit tests checking HTML parsing code for 'mbed.htm' files
    """

    details_txt_0226 = """Version: 0226
Build:   Aug 24 2015 17:06:30
Git Commit SHA: 27a236b9fe39c674a703c5c89655fbd26b8e27e1
Git Local mods: Yes
"""

    details_txt_0240 = """# DAPLink Firmware - see https://mbed.com/daplink
Unique ID: 0240000029164e45002f0012706e0006f301000097969900
HIF ID: 97969900
Auto Reset: 0
Automation allowed: 0
Daplink Mode: Interface
Interface Version: 0240
Git SHA: c765cbb590f57598756683254ca38b211693ae5e
Local Mods: 0
USB Interfaces: MSD, CDC, HID
Interface CRC: 0x26764ebf
"""

    def setUp(self):
        self.mbeds = create()

    def tearDown(self):
        pass

    def test_simplified_daplink_txt_content(self):
        # Fetch lines from DETAILS.TXT
        lines = self.details_txt_0226.splitlines()
        self.assertEqual(4, len(lines))

        # Check parsing content
        result = self.mbeds._parse_details(lines)
        self.assertEqual(4, len(result))
        self.assertIn('Version', result)
        self.assertIn('Build', result)
        self.assertIn('Git Commit SHA', result)
        self.assertIn('Git Local mods', result)

        # Check for daplink_version
        self.assertEqual(result['Version'], "0226")

    def test_extended_daplink_txt_content(self):
        # Fetch lines from DETAILS.TXT
        lines = self.details_txt_0240.splitlines()
        self.assertEqual(11, len(lines))

        # Check parsing content
        result = self.mbeds._parse_details(lines)
        self.assertEqual(11, len(result))   # 12th would be comment
        self.assertIn('Unique ID', result)
        self.assertIn('HIF ID', result)
        self.assertIn('Auto Reset', result)
        self.assertIn('Automation allowed', result)
        self.assertIn('Daplink Mode', result)
        self.assertIn('Interface Version', result)
        self.assertIn('Git SHA', result)
        self.assertIn('Local Mods', result)
        self.assertIn('USB Interfaces', result)
        self.assertIn('Interface CRC', result)

        # Check if we parsed comment line:
        # "# DAPLink Firmware - see https://mbed.com/daplink"
        for key in result:
            # Check if we parsed comment
            self.assertFalse(key.startswith('#'))
            # Check if we parsed
            self.assertFalse('https://mbed.com/daplink' in result[key])

        # Check for daplink_version
        # DAPlink <240 compatibility
        self.assertEqual(result['Interface Version'], "0240")
        self.assertEqual(result['Version'], "0240")

    def test_(self):
        pass

    def test_(self):
        pass

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
'''
mbed SDK
Copyright (c) 2011-2015 ARM Limited

Licensed under the Apache License, Version 2.0 (the 'License');
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an 'AS IS' BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
'''

import unittest
from mock import patch
from mbed_lstools.linux import MbedLsToolsLinuxGeneric


class LinuxPortTestCase(unittest.TestCase):
    ''' Basic test cases checking trivial asserts
    '''

    def setUp(self):
        self.linux_generic = MbedLsToolsLinuxGeneric()

    def tearDown(self):
        pass

    vfat_devices = [
        b'/dev/sdb on /media/usb0 type vfat (rw,noexec,nodev,sync,noatime,nodiratime,gid=1000,uid=1000,dmask=000,fmask=000)',
        b'/dev/sdd on /media/usb2 type vfat (rw,noexec,nodev,sync,noatime,nodiratime,gid=1000,uid=1000,dmask=000,fmask=000)',
        b'/dev/sde on /media/usb3 type vfat (rw,noexec,nodev,sync,noatime,nodiratime,gid=1000,uid=1000,dmask=000,fmask=000)',
        b'/dev/sdc on /media/usb1 type vfat (rw,noexec,nodev,sync,noatime,nodiratime,gid=1000,uid=1000,dmask=000,fmask=000)'
    ]

    def test_get_mount_point_basic(self):
        with patch('mbed_lstools.linux.MbedLsToolsLinuxGeneric._run_cli_process') as _cliproc:
            _cliproc.return_value = (b'\n'.join(self.vfat_devices), None, 0)
            mount_dict = dict(self.linux_generic._fat_mounts())
            _cliproc.assert_called_once_with('mount')
        self.assertEqual('/media/usb0', mount_dict['/dev/sdb'])
        self.assertEqual('/media/usb2', mount_dict['/dev/sdd'])
        self.assertEqual('/media/usb3', mount_dict['/dev/sde'])
        self.assertEqual('/media/usb1', mount_dict['/dev/sdc'])


    vfat_devices_ext = [
        b'/dev/sdb on /media/MBED_xxx type vfat (rw,noexec,nodev,sync,noatime,nodiratime,gid=1000,uid=1000,dmask=000,fmask=000)',
        b'/dev/sdd on /media/MBED___x type vfat (rw,noexec,nodev,sync,noatime,nodiratime,gid=1000,uid=1000,dmask=000,fmask=000)',
        b'/dev/sde on /media/MBED-xxx type vfat (rw,noexec,nodev,sync,noatime,nodiratime,gid=1000,uid=1000,dmask=000,fmask=000)',
        b'/dev/sdc on /media/MBED_x-x type vfat (rw,noexec,nodev,sync,noatime,nodiratime,gid=1000,uid=1000,dmask=000,fmask=000)',

        b'/dev/sda on /mnt/NUCLEO type vfat (rw,relatime,uid=999,fmask=0133,dmask=0022,codepage=437,iocharset=ascii,shortname=mixed,utf8,flush,errors=remount-ro,uhelper=ldm)',
        b'/dev/sdf on /mnt/NUCLEO_ type vfat (rw,relatime,uid=999,fmask=0133,dmask=0022,codepage=437,iocharset=ascii,shortname=mixed,utf8,flush,errors=remount-ro,uhelper=ldm)',
        b'/dev/sdg on /mnt/DAPLINK type vfat (rw,relatime,sync,uid=999,fmask=0022,dmask=0022,codepage=437,iocharset=ascii,shortname=mixed,errors=remount-ro,uhelper=ldm)',
        b'/dev/sdh on /mnt/DAPLINK_ type vfat (rw,relatime,sync,uid=999,fmask=0022,dmask=0022,codepage=437,iocharset=ascii,shortname=mixed,errors=remount-ro,uhelper=ldm)',
        b'/dev/sdi on /mnt/DAPLINK__ type vfat (rw,relatime,sync,uid=999,fmask=0022,dmask=0022,codepage=437,iocharset=ascii,shortname=mixed,errors=remount-ro,uhelper=ldm)',
    ]

    def test_get_mount_point_ext(self):
        with patch('mbed_lstools.linux.MbedLsToolsLinuxGeneric._run_cli_process') as _cliproc:
            _cliproc.return_value = (b'\n'.join(self.vfat_devices_ext), None, 0)
            mount_dict = dict(self.linux_generic._fat_mounts())
            _cliproc.assert_called_once_with('mount')
        self.assertEqual('/media/MBED_xxx', mount_dict['/dev/sdb'])
        self.assertEqual('/media/MBED___x', mount_dict['/dev/sdd'])
        self.assertEqual('/media/MBED-xxx', mount_dict['/dev/sde'])
        self.assertEqual('/media/MBED_x-x', mount_dict['/dev/sdc'])

        self.assertEqual('/mnt/NUCLEO', mount_dict['/dev/sda'])
        self.assertEqual('/mnt/NUCLEO_', mount_dict['/dev/sdf'])
        self.assertEqual('/mnt/DAPLINK', mount_dict['/dev/sdg'])
        self.assertEqual('/mnt/DAPLINK_', mount_dict['/dev/sdh'])
        self.assertEqual('/mnt/DAPLINK__', mount_dict['/dev/sdi'])

    def find_candidates_with_patch(self, mount_list, link_dict, listdir_dict):
        with patch('mbed_lstools.linux.MbedLsToolsLinuxGeneric._run_cli_process') as _cliproc,\
             patch('os.readlink') as _readlink,\
             patch('os.listdir') as _listdir,\
             patch('mbed_lstools.linux.isdir') as _isdir:
            _isdir.return_value = True
            _cliproc.return_value = (b'\n'.join(mount_list), None, 0)
            def do_readlink(link):
                return link_dict[link]
            _readlink.side_effect = do_readlink
            def do_listdir(dir):
                return listdir_dict[dir]
            _listdir.side_effect = do_listdir
            ret_val = self.linux_generic.find_candidates()
            _cliproc.assert_called_once_with('mount')
            return ret_val


    listdir_dict_rpi = {
        '/dev/disk/by-id': [
            'usb-MBED_VFS_0240000028634e4500135006691700105f21000097969900-0:0',
            'usb-MBED_VFS_0240000028884e450018700f6bf000338021000097969900-0:0',
            'usb-MBED_VFS_0240000028884e45001f700f6bf000118021000097969900-0:0',
            'usb-MBED_VFS_0240000028884e450036700f6bf000118021000097969900-0:0',
            'usb-MBED_VFS_0240000029164e45001b0012706e000df301000097969900-0:0',
            'usb-MBED_VFS_0240000029164e45002f0012706e0006f301000097969900-0:0'
        ],
        '/dev/serial/by-id': [
            'usb-ARM_DAPLink_CMSIS-DAP_0240000028634e4500135006691700105f21000097969900-if01',
            'usb-ARM_DAPLink_CMSIS-DAP_0240000028884e450018700f6bf000338021000097969900-if01',
            'usb-ARM_DAPLink_CMSIS-DAP_0240000028884e450036700f6bf000118021000097969900-if01',
            'usb-ARM_DAPLink_CMSIS-DAP_0240000029164e45001b0012706e000df301000097969900-if01'
        ]
    }

    link_dict_rpi = {
        '/dev/disk/by-id/usb-MBED_VFS_0240000028634e4500135006691700105f21000097969900-0:0': '../../sdb',
        '/dev/disk/by-id/usb-MBED_VFS_0240000028884e450018700f6bf000338021000097969900-0:0': '../../sdc',
        '/dev/disk/by-id/usb-MBED_VFS_0240000028884e45001f700f6bf000118021000097969900-0:0': '../../sdd',
        '/dev/disk/by-id/usb-MBED_VFS_0240000028884e450036700f6bf000118021000097969900-0:0': '../../sde',
        '/dev/disk/by-id/usb-MBED_VFS_0240000029164e45001b0012706e000df301000097969900-0:0': '../../sdf',
        '/dev/disk/by-id/usb-MBED_VFS_0240000029164e45002f0012706e0006f301000097969900-0:0': '../../sdg',
        '/dev/serial/by-id/usb-ARM_DAPLink_CMSIS-DAP_0240000028634e4500135006691700105f21000097969900-if01': '../../ttyACM0',
        '/dev/serial/by-id/usb-ARM_DAPLink_CMSIS-DAP_0240000028884e450018700f6bf000338021000097969900-if01': '../../ttyACM1',
        '/dev/serial/by-id/usb-ARM_DAPLink_CMSIS-DAP_0240000028884e450036700f6bf000118021000097969900-if01': '../../ttyACM3',
        '/dev/serial/by-id/usb-ARM_DAPLink_CMSIS-DAP_0240000029164e45001b0012706e000df301000097969900-if01': '../../ttyACM2'
    }

    mount_list_rpi = [
        b'/dev/sdb on /media/usb0 type vfat (rw,noexec,nodev,sync,noatime,nodiratime,gid=1000,uid=1000,dmask=000,fmask=000)',
        b'/dev/sdc on /media/usb1 type vfat (rw,noexec,nodev,sync,noatime,nodiratime,gid=1000,uid=1000,dmask=000,fmask=000)',
        b'/dev/sdd on /media/usb2 type vfat (rw,noexec,nodev,sync,noatime,nodiratime,gid=1000,uid=1000,dmask=000,fmask=000)',
        b'/dev/sde on /media/usb3 type vfat (rw,noexec,nodev,sync,noatime,nodiratime,gid=1000,uid=1000,dmask=000,fmask=000)',
        b'/dev/sdf on /media/usb4 type vfat (rw,noexec,nodev,sync,noatime,nodiratime,gid=1000,uid=1000,dmask=000,fmask=000)',
        b'/dev/sdg on /media/usb5 type vfat (rw,noexec,nodev,sync,noatime,nodiratime,gid=1000,uid=1000,dmask=000,fmask=000)'
    ]
    def test_get_detected_rpi(self):
        mbed_det = self.find_candidates_with_patch(
            self.mount_list_rpi, self.link_dict_rpi, self.listdir_dict_rpi)

        self.assertIn({
            'mount_point': '/media/usb0',
            'serial_port': '/dev/ttyACM0',
            'target_id_usb_id': '0240000028634e4500135006691700105f21000097969900'
          }, mbed_det)
        self.assertIn({
            'mount_point': '/media/usb1',
            'serial_port': '/dev/ttyACM1',
            'target_id_usb_id': '0240000028884e450018700f6bf000338021000097969900'
          }, mbed_det)
        self.assertIn({
            'mount_point': '/media/usb4',
            'serial_port': '/dev/ttyACM2',
            'target_id_usb_id': '0240000029164e45001b0012706e000df301000097969900'
          }, mbed_det)

        self.assertIn({
            'mount_point': '/media/usb3',
            'serial_port': '/dev/ttyACM3',
            'target_id_usb_id': '0240000028884e450036700f6bf000118021000097969900'
          }, mbed_det)


    listdir_dict_1 = {
        '/dev/disk/by-id': [
            '/dev/disk/by-id/ata-HDS728080PLA380_40Y9028LEN_PFDB32S7S44XLM',
            '/dev/disk/by-id/ata-HDS728080PLA380_40Y9028LEN_PFDB32S7S44XLM-part1',
            '/dev/disk/by-id/ata-HDS728080PLA380_40Y9028LEN_PFDB32S7S44XLM-part2',
            '/dev/disk/by-id/ata-HDS728080PLA380_40Y9028LEN_PFDB32S7S44XLM-part5',
            '/dev/disk/by-id/ata-TSSTcorpDVD-ROM_TS-H352C',
            '/dev/disk/by-id/usb-MBED_FDi_sk_A000000001-0:0',
            '/dev/disk/by-id/usb-MBED_microcontroller_0240020152986E5EAF6693E6-0:0',
            '/dev/disk/by-id/wwn-0x5000cca30ccffb77',
            '/dev/disk/by-id/wwn-0x5000cca30ccffb77-part1',
            '/dev/disk/by-id/wwn-0x5000cca30ccffb77-part2',
            '/dev/disk/by-id/wwn-0x5000cca30ccffb77-part5',
        ],
        '/dev/serial/by-id': [
            '/dev/serial/by-id/usb-MBED_MBED_CMSIS-DAP_0240020152986E5EAF6693E6-if01',
            '/dev/serial/by-id/usb-MBED_MBED_CMSIS-DAP_A000000001-if01',
        ]
    }

    link_dict_1 = {
        '/dev/disk/by-id/ata-HDS728080PLA380_40Y9028LEN_PFDB32S7S44XLM': '../../sda',
        '/dev/disk/by-id/ata-HDS728080PLA380_40Y9028LEN_PFDB32S7S44XLM-part1': '../../sda1',
        '/dev/disk/by-id/ata-HDS728080PLA380_40Y9028LEN_PFDB32S7S44XLM-part2': '../../sda2',
        '/dev/disk/by-id/ata-HDS728080PLA380_40Y9028LEN_PFDB32S7S44XLM-part5': '../../sda5',
        '/dev/disk/by-id/ata-TSSTcorpDVD-ROM_TS-H352C': '../../sr0',
        '/dev/disk/by-id/usb-MBED_FDi_sk_A000000001-0:0': '../../sdc',
        '/dev/disk/by-id/usb-MBED_microcontroller_0240020152986E5EAF6693E6-0:0': '../../sdb',
        '/dev/disk/by-id/wwn-0x5000cca30ccffb77': '../../sda',
        '/dev/disk/by-id/wwn-0x5000cca30ccffb77-part1': '../../sda1',
        '/dev/disk/by-id/wwn-0x5000cca30ccffb77-part2': '../../sda2',
        '/dev/disk/by-id/wwn-0x5000cca30ccffb77-part5': '../../sda5',
        '/dev/serial/by-id/usb-MBED_MBED_CMSIS-DAP_0240020152986E5EAF6693E6-if01': '../../ttyACM1',
        '/dev/serial/by-id/usb-MBED_MBED_CMSIS-DAP_A000000001-if01': '../../ttyACM0'
    }

    mount_list_1 = [
        b'/dev/sdb on /media/usb0 type vfat (rw,noexec,nodev,sync,noatime,nodiratime,gid=1000,uid=1000,dmask=000,fmask=000)',
        b'/dev/sdc on /media/usb1 type vfat (rw,noexec,nodev,sync,noatime,nodiratime,gid=1000,uid=1000,dmask=000,fmask=000)'
    ]
    def test_get_detected_1_k64f(self):
        mbed_det = self.find_candidates_with_patch(
            self.mount_list_1, self.link_dict_1, self.listdir_dict_1)
        self.assertIn({
            'mount_point': '/media/usb0',
            'serial_port': '/dev/ttyACM1',
            'target_id_usb_id': '0240020152986E5EAF6693E6'
          }, mbed_det)

        self.assertIn({
            'mount_point': '/media/usb1',
            'serial_port': '/dev/ttyACM0',
            'target_id_usb_id': 'A000000001'
          }, mbed_det)


    listdir_dict_2 = {
        '/dev/disk/by-id': [
            'ata-HDS728080PLA380_40Y9028LEN_PFDB32S7S44XLM',
            'ata-HDS728080PLA380_40Y9028LEN_PFDB32S7S44XLM-part1',
            'ata-HDS728080PLA380_40Y9028LEN_PFDB32S7S44XLM-part2',
            'ata-HDS728080PLA380_40Y9028LEN_PFDB32S7S44XLM-part5',
            'ata-TSSTcorpDVD-ROM_TS-H352C',
            'usb-MBED_FDi_sk_A000000001-0:0',
            'usb-MBED_microcontroller_02400201489A1E6CB564E3D4-0:0',
            'usb-MBED_microcontroller_0240020152986E5EAF6693E6-0:0',
            'usb-MBED_microcontroller_0240020152A06E54AF5E93EC-0:0',
            'usb-MBED_microcontroller_0672FF485649785087171742-0:0',
            'wwn-0x5000cca30ccffb77',
            'wwn-0x5000cca30ccffb77-part1',
            'wwn-0x5000cca30ccffb77-part2',
            'wwn-0x5000cca30ccffb77-part5'
        ],
        '/dev/serial/by-id': [
            'usb-MBED_MBED_CMSIS-DAP_02400201489A1E6CB564E3D4-if01',
            'usb-MBED_MBED_CMSIS-DAP_0240020152986E5EAF6693E6-if01',
            'usb-MBED_MBED_CMSIS-DAP_0240020152A06E54AF5E93EC-if01',
            'usb-MBED_MBED_CMSIS-DAP_A000000001-if01',
            'usb-STMicroelectronics_STM32_STLink_0672FF485649785087171742-if02'
        ]
    }

    link_dict_2 = {
        '/dev/disk/by-id/ata-HDS728080PLA380_40Y9028LEN_PFDB32S7S44XLM': '../../sda',
        '/dev/disk/by-id/ata-HDS728080PLA380_40Y9028LEN_PFDB32S7S44XLM-part1': '../../sda1',
        '/dev/disk/by-id/ata-HDS728080PLA380_40Y9028LEN_PFDB32S7S44XLM-part2': '../../sda2',
        '/dev/disk/by-id/ata-HDS728080PLA380_40Y9028LEN_PFDB32S7S44XLM-part5': '../../sda5',
        '/dev/disk/by-id/ata-TSSTcorpDVD-ROM_TS-H352C': '../../sr0',
        '/dev/disk/by-id/usb-MBED_FDi_sk_A000000001-0:0': '../../sdc',
        '/dev/disk/by-id/usb-MBED_microcontroller_02400201489A1E6CB564E3D4-0:0': '../../sde',
        '/dev/disk/by-id/usb-MBED_microcontroller_0240020152986E5EAF6693E6-0:0': '../../sdb',
        '/dev/disk/by-id/usb-MBED_microcontroller_0240020152A06E54AF5E93EC-0:0': '../../sdf',
        '/dev/disk/by-id/usb-MBED_microcontroller_0672FF485649785087171742-0:0': '../../sdd',
        '/dev/disk/by-id/wwn-0x5000cca30ccffb77': '../../sda',
        '/dev/disk/by-id/wwn-0x5000cca30ccffb77-part1': '../../sda1',
        '/dev/disk/by-id/wwn-0x5000cca30ccffb77-part2': '../../sda2',
        '/dev/disk/by-id/wwn-0x5000cca30ccffb77-part5': '../../sda5',
        '/dev/serial/by-id/usb-MBED_MBED_CMSIS-DAP_02400201489A1E6CB564E3D4-if01': '../../ttyACM3',
        '/dev/serial/by-id/usb-MBED_MBED_CMSIS-DAP_0240020152986E5EAF6693E6-if01': '../../ttyACM1',
        '/dev/serial/by-id/usb-MBED_MBED_CMSIS-DAP_0240020152A06E54AF5E93EC-if01': '../../ttyACM4',
        '/dev/serial/by-id/usb-MBED_MBED_CMSIS-DAP_A000000001-if01': '../../ttyACM0',
        '/dev/serial/by-id/usb-STMicroelectronics_STM32_STLink_0672FF485649785087171742-if02': '../../ttyACM2'
    }

    mount_list_2 = [
        b'/dev/sdb on /media/usb0 type vfat (rw,noexec,nodev,sync,noatime,nodiratime,gid=1000,uid=1000,dmask=000,fmask=000)',
        b'/dev/sdc on /media/usb1 type vfat (rw,noexec,nodev,sync,noatime,nodiratime,gid=1000,uid=1000,dmask=000,fmask=000)',
        b'/dev/sdd on /media/usb2 type vfat (rw,noexec,nodev,sync,noatime,nodiratime,gid=1000,uid=1000,dmask=000,fmask=000)',
        b'/dev/sde on /media/usb3 type vfat (rw,noexec,nodev,sync,noatime,nodiratime,gid=1000,uid=1000,dmask=000,fmask=000)',
        b'/dev/sdf on /media/usb4 type vfat (rw,noexec,nodev,sync,noatime,nodiratime,gid=1000,uid=1000,dmask=000,fmask=000)'
    ]
    def test_get_detected_2_k64f(self):
        mbed_det = self.find_candidates_with_patch(
            self.mount_list_2, self.link_dict_2, self.listdir_dict_2)

        self.assertIn({
            'mount_point': '/media/usb1',
            'serial_port': '/dev/ttyACM0',
            'target_id_usb_id': 'A000000001'
          },
          mbed_det)
        self.assertIn({
            'mount_point': '/media/usb2',
            'serial_port': '/dev/ttyACM2',
            'target_id_usb_id': '0672FF485649785087171742'
          },
          mbed_det)

        self.assertIn({
            'mount_point': '/media/usb4',
            'serial_port': '/dev/ttyACM4',
            'target_id_usb_id': '0240020152A06E54AF5E93EC'
          },
          mbed_det)

        self.assertIn({
            'mount_point': '/media/usb3',
            'serial_port': '/dev/ttyACM3',
            'target_id_usb_id': '02400201489A1E6CB564E3D4'
          },
          mbed_det)

        self.assertIn({
            'mount_point': '/media/usb0',
            'serial_port': '/dev/ttyACM1',
            'target_id_usb_id': '0240020152986E5EAF6693E6'
          },
          mbed_det)


    listdir_dict_4 = {
        '/dev/disk/by-id': [
            'ata-VMware_Virtual_SATA_CDRW_Drive_00000000000000000001',
            'ata-VMware_Virtual_SATA_CDRW_Drive_01000000000000000001',
            'usb-MBED_VFS_0240000033514e45001f500585d40014e981000097969900-0:0'
        ],
        '/dev/serial/by-id': [
            'pci-ARM_DAPLink_CMSIS-DAP_0240000033514e45001f500585d40014e981000097969900-if01'
        ]
    }

    link_dict_4 = {
        '/dev/disk/by-id/ata-VMware_Virtual_SATA_CDRW_Drive_00000000000000000001': '../../sr0',
        '/dev/disk/by-id/ata-VMware_Virtual_SATA_CDRW_Drive_01000000000000000001': '../../sr1',
        '/dev/disk/by-id/usb-MBED_VFS_0240000033514e45001f500585d40014e981000097969900-0:0': '../../sdb',
        '/dev/serial/by-id/pci-ARM_DAPLink_CMSIS-DAP_0240000033514e45001f500585d40014e981000097969900-if01': '../../ttyACM0'
    }

    mount_list_4 = [
        b'/dev/sdb on /media/przemek/DAPLINK type vfat (rw,nosuid,nodev,relatime,uid=1000,gid=1000,fmask=0022,dmask=0022,codepage=437,iocharset=iso8859-1,shortname=mixed,showexec,utf8,flush,errors=remount-ro,uhelper=udisks2)'
    ]
    def test_get_detected_3_k64f(self):
        mbed_det = self.find_candidates_with_patch(
            self.mount_list_4, self.link_dict_4, self.listdir_dict_4)

        self.assertIn({
            'mount_point': '/media/przemek/DAPLINK',
            'serial_port': '/dev/ttyACM0',
            'target_id_usb_id': '0240000033514e45001f500585d40014e981000097969900'
          },
          mbed_det)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""
mbed SDK
Copyright (c) 2011-2015 ARM Limited

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import unittest
import os
import errno
import logging
import re
from mock import patch
from copy import deepcopy

from mbed_lstools.lstools_base import MbedLsToolsBase, FSInteraction

class DummyLsTools(MbedLsToolsBase):
    return_value = []
    def find_candidates(self):
        return self.return_value

try:
    basestring
except NameError:
    # Python 3
    basestring = str

class BasicTestCase(unittest.TestCase):
    """ Basic test cases checking trivial asserts
    """

    def setUp(self):
        self.base = DummyLsTools()

    def tearDown(self):
        pass

    def test_list_mbeds(self):
        self.base.return_value = [{'mount_point': 'dummy_mount_point',
                                   'target_id_usb_id': u'0240DEADBEEF',
                                   'serial_port': "dummy_serial_port"},
                                  {'mount_point': None,
                                   'target_id_usb_id': '00000000000',
                                   'serial_port': 'not_valid'}]
        with patch("mbed_lstools.lstools_base.MbedLsToolsBase._read_htm_ids") as _read_htm,\
             patch("mbed_lstools.lstools_base.MbedLsToolsBase.mount_point_ready") as _mpr,\
             patch("mbed_lstools.lstools_base.PlatformDatabase.get") as _get:
            _mpr.return_value = True
            _read_htm.return_value = (u'0241BEEFDEAD', {})
            _get.return_value = 'foo_target'
            to_check = self.base.list_mbeds()
            _read_htm.assert_called_once_with('dummy_mount_point')
            _get.assert_called_once_with('0241')
        self.assertEqual(len(to_check), 1)
        self.assertEqual(to_check[0]['target_id'], "0241BEEFDEAD")
        self.assertEqual(to_check[0]['platform_name'], 'foo_target')

    def test_list_mbeds(self):
        self.base.return_value = [{'mount_point': 'dummy_mount_point',
                                   'target_id_usb_id': u'not_in_target_db',
                                   'serial_port': "dummy_serial_port"}]
        for qos in [FSInteraction.BeforeFilter, FSInteraction.AfterFilter]:
            with patch("mbed_lstools.lstools_base.MbedLsToolsBase._read_htm_ids") as _read_htm,\
                patch("mbed_lstools.lstools_base.MbedLsToolsBase.mount_point_ready") as _mpr,\
                patch("mbed_lstools.lstools_base.PlatformDatabase.get") as _get:
                _mpr.return_value = True
                _read_htm.return_value = (u'not_in_target_db', {})
                _get.return_value = None
                to_check = self.base.list_mbeds()
                _read_htm.assert_called_once_with('dummy_mount_point')
                _get.assert_called_once_with('not_')
            self.assertEqual(len(to_check), 1)
            self.assertEqual(to_check[0]['target_id'], "not_in_target_db")
            self.assertEqual(to_check[0]['platform_name'], None)

    def test_list_manufacture_ids(self):
        table_str = self.base.list_manufacture_ids()
        self.assertTrue(isinstance(table_str, basestring))

    def test_mock_manufacture_ids_default_multiple(self):
        # oper='+'
        for mid, platform_name in [('0341', 'TEST_PLATFORM_NAME_1'),
                                   ('0342', 'TEST_PLATFORM_NAME_2'),
                                   ('0343', 'TEST_PLATFORM_NAME_3')]:
            self.base.mock_manufacture_id(mid, platform_name)
            self.assertEqual(platform_name, self.base.plat_db.get(mid))

    def test_mock_manufacture_ids_minus(self):
        # oper='+'
        for mid, platform_name in [('0341', 'TEST_PLATFORM_NAME_1'),
                                   ('0342', 'TEST_PLATFORM_NAME_2'),
                                   ('0343', 'TEST_PLATFORM_NAME_3')]:
            self.base.mock_manufacture_id(mid, platform_name)
            self.assertEqual(platform_name, self.base.plat_db.get(mid))

        # oper='-'
        mock_ids = self.base.mock_manufacture_id('0342', '', oper='-')
        self.assertEqual('TEST_PLATFORM_NAME_1', self.base.plat_db.get("0341"))
        self.assertEqual(None, self.base.plat_db.get("0342"))
        self.assertEqual('TEST_PLATFORM_NAME_3', self.base.plat_db.get("0343"))

    def test_mock_manufacture_ids_star(self):
        # oper='+'
        for mid, platform_name in [('0341', 'TEST_PLATFORM_NAME_1'),
                                   ('0342', 'TEST_PLATFORM_NAME_2'),
                                   ('0343', 'TEST_PLATFORM_NAME_3')]:
            self.base.mock_manufacture_id(mid, platform_name)

            self.assertEqual(platform_name, self.base.plat_db.get(mid))

        # oper='-'
        self.base.mock_manufacture_id('*', '', oper='-')
        self.assertEqual(None, self.base.plat_db.get("0341"))
        self.assertEqual(None, self.base.plat_db.get("0342"))
        self.assertEqual(None, self.base.plat_db.get("0343"))

    def test_fs_never(self):
        device = {
            'target_id_usb_id': '024075309420ABCE',
            'mount_point': 'invalid_mount_point',
            'serial_port': 'invalid_serial_port'
        }
        with patch("mbed_lstools.lstools_base.MbedLsToolsBase._update_device_from_htm") as _up_htm:
            filter = re.compile("")
            ret = self.base._fs_never(device, filter)
            self.assertIsNotNone(ret)
            self.assertEqual(ret['target_id'], ret['target_id_usb_id'])
            _up_htm.assert_not_called()

            filter_out = re.compile("NOT-K64F")
            ret = self.base._fs_never(device, filter_out)
            self.assertIsNone(ret)
            _up_htm.assert_not_called()

    def test_fs_after(self):
        device = {
            'target_id_usb_id': '024075309420ABCE',
            'mount_point': 'invalid_mount_point',
            'serial_port': 'invalid_serial_port'
        }
        with patch("mbed_lstools.lstools_base.MbedLsToolsBase._read_htm_ids") as _read_htm:
            new_device_id = "00017531642046"
            _read_htm.return_value = (new_device_id, {})
            filter = re.compile("")
            ret = self.base._fs_after_id_check(device, filter)
            self.assertIsNotNone(ret)
            self.assertEqual(ret['target_id'], new_device_id)
            _read_htm.assert_called_with(device['mount_point'])

            _read_htm.reset_mock()

            filter_out = re.compile("NOT-K64F")
            ret = self.base._fs_after_id_check(device, filter_out)
            self.assertIsNone(ret)
            _read_htm.assert_not_called()

    def test_fs_before(self):
        device = {
            'target_id_usb_id': '024075309420ABCE',
            'mount_point': 'invalid_mount_point',
            'serial_port': 'invalid_serial_port'
        }
        with patch("mbed_lstools.lstools_base.MbedLsToolsBase._read_htm_ids") as _read_htm:
            new_device_id = u'00017575430420'
            _read_htm.return_value = (new_device_id, {})
            filter = re.compile(u'')
            ret = self.base._fs_before_id_check(device, filter)
            self.assertIsNotNone(ret)
            self.assertEqual(ret['target_id'], new_device_id)
            _read_htm.assert_called_with(device['mount_point'])

            _read_htm.reset_mock()

            filter_out = re.compile("NOT-LPC2368")
            ret = self.base._fs_before_id_check(device, filter_out)
            self.assertIsNone(ret)
            _read_htm.assert_called_with(device['mount_point'])

if __name__ == '__main__':
    unittest.main()



"""
mbed SDK
Copyright (c) 2017 ARM Limited

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

"""Functions that manage a platform database"""

import json
import re
from collections import OrderedDict
from copy import copy
from io import open
from os import makedirs
from os.path import join, dirname
from appdirs import user_data_dir
from fasteners import InterProcessLock

try:
    unicode
except NameError:
    unicode = str

import logging
logger = logging.getLogger("mbedls.platform_database")
del logging

LOCAL_PLATFORM_DATABASE = join(user_data_dir("mbedls"), "platforms.json")
LOCAL_MOCKS_DATABASE = join(user_data_dir("mbedls"), "mock.json")

DEFAULT_PLATFORM_DB = {
    u'0001': u'LPC2368',
    u'0002': u'LPC2368',
    u'0003': u'LPC2368',
    u'0004': u'LPC2368',
    u'0005': u'LPC2368',
    u'0006': u'LPC2368',
    u'0007': u'LPC2368',
    u'0100': u'LPC2368',
    u'0183': u'UBLOX_C027',
    u'0200': u'KL25Z',
    u'0201': u'KW41Z',
    u'0210': u'KL05Z',
    u'0214': u'HEXIWEAR',
    u'0217': u'K82F',
    u'0218': u'KL82Z',
    u'0220': u'KL46Z',
    u'0230': u'K20D50M',
    u'0231': u'K22F',
    u'0240': u'K64F',
    u'0245': u'K64F',
    u'0250': u'KW24D',
    u'0261': u'KL27Z',
    u'0262': u'KL43Z',
    u'0300': u'MTS_GAMBIT',
    u'0305': u'MTS_MDOT_F405RG',
    u'0310': u'MTS_DRAGONFLY_F411RE',
    u'0311': u'K66F',
    u'0315': u'MTS_MDOT_F411RE',
    u'0350': u'XDOT_L151CC',
    u'0400': u'MAXWSNENV',
    u'0405': u'MAX32600MBED',
    u'0406': u'MAX32620MBED',
    u'0407': u'MAX32620HSP',
    u'0408': u'MAX32625NEXPAQ',
    u'0409': u'MAX32630FTHR',
    u'0415': u'MAX32625MBED',
    u'0500': u'SPANSION_PLACEHOLDER',
    u'0505': u'SPANSION_PLACEHOLDER',
    u'0510': u'SPANSION_PLACEHOLDER',
    u'0602': u'EV_COG_AD3029LZ',
    u'0603': u'EV_COG_AD4050LZ',
    u'0700': u'NUCLEO_F103RB',
    u'0705': u'NUCLEO_F302R8',
    u'0710': u'NUCLEO_L152RE',
    u'0715': u'NUCLEO_L053R8',
    u'0720': u'NUCLEO_F401RE',
    u'0725': u'NUCLEO_F030R8',
    u'0730': u'NUCLEO_F072RB',
    u'0735': u'NUCLEO_F334R8',
    u'0740': u'NUCLEO_F411RE',
    u'0743': u'DISCO_F413ZH',
    u'0744': u'NUCLEO_F410RB',
    u'0745': u'NUCLEO_F303RE',
    u'0747': u'NUCLEO_F303ZE',
    u'0750': u'NUCLEO_F091RC',
    u'0755': u'NUCLEO_F070RB',
    u'0760': u'NUCLEO_L073RZ',
    u'0764': u'DISCO_L475VG_IOT01A',
    u'0765': u'NUCLEO_L476RG',
    u'0766': u'SILICA_SENSOR_NODE',
    u'0770': u'NUCLEO_L432KC',
    u'0775': u'NUCLEO_F303K8',
    u'0777': u'NUCLEO_F446RE',
    u'0778': u'NUCLEO_F446ZE',
    u'0779': u'NUCLEO_L433RC_P',
    u'0780': u'NUCLEO_L011K4',
    u'0785': u'NUCLEO_F042K6',
    u'0788': u'DISCO_F469NI',
    u'0790': u'NUCLEO_L031K6',
    u'0791': u'NUCLEO_F031K6',
    u'0795': u'DISCO_F429ZI',
    u'0796': u'NUCLEO_F429ZI',
    u'0797': u'NUCLEO_F439ZI',
    u'0799': u'ST_PLACEHOLDER',
    u'0805': u'DISCO_L053C8',
    u'0810': u'DISCO_F334C8',
    u'0812': u'NUCLEO_F722ZE',
    u'0815': u'DISCO_F746NG',
    u'0816': u'NUCLEO_F746ZG',
    u'0817': u'DISCO_F769NI',
    u'0818': u'NUCLEO_F767ZI',
    u'0819': u'NUCLEO_F756ZG',
    u'0820': u'DISCO_L476VG',
    u'0821': u'NUCLEO_L452RE',
    u'0823': u'NUCLEO_L496ZG',
    u'0824': u'LPC824',
    u'0826': u'NUCLEO_F412ZG',
    u'0827': u'NUCLEO_L486RG',
    u'0828': u'NUCLEO_L496ZG_P',
    u'0829': u'NUCLEO_L452RE_P',
    u'0833': u'DISCO_L072CZ_LRWAN1',
    u'0835': u'NUCLEO_F207ZG',
    u'0840': u'B96B_F446VE',
    u'0900': u'XPRO_SAMR21',
    u'0905': u'XPRO_SAMW25',
    u'0910': u'XPRO_SAML21',
    u'0915': u'XPRO_SAMD21',
    u'1000': u'LPC2368',
    u'1001': u'LPC2368',
    u'1010': u'LPC1768',
    u'1017': u'HRM1017',
    u'1018': u'SSCI824',
    u'1019': u'TY51822R3',
    u'1022': u'RO359B',
    u'1034': u'LPC11U34',
    u'1040': u'LPC11U24',
    u'1045': u'LPC11U24',
    u'1050': u'LPC812',
    u'1054': u'LPC54114',
    u'1056': u'LPC546XX',
    u'1060': u'LPC4088',
    u'1061': u'LPC11U35_401',
    u'1062': u'LPC4088_DM',
    u'1070': u'NRF51822',
    u'1075': u'NRF51822_OTA',
    u'1080': u'OC_MBUINO',
    u'1090': u'RBLAB_NRF51822',
    u'1095': u'RBLAB_BLENANO',
    u'1100': u'NRF51_DK',
    u'1101': u'NRF52_DK',
    u'1102': u'NRF52840_DK',
    u'1105': u'NRF51_DK_OTA',
    u'1114': u'LPC1114',
    u'1120': u'NRF51_DONGLE',
    u'1130': u'NRF51822_SBK',
    u'1140': u'WALLBOT_BLE',
    u'1168': u'LPC11U68',
    u'1200': u'NCS36510',
    u'1234': u'UBLOX_C027',
    u'1235': u'UBLOX_C027',
    u'1236': u'UBLOX_EVK_ODIN_W2',
    u'1237': u'UBLOX_EVK_NINA_B1',
    u'1300': u'NUC472-NUTINY',
    u'1301': u'NUMBED',
    u'1302': u'NUMAKER_PFM_NUC472',
    u'1303': u'NUMAKER_PFM_M453',
    u'1304': u'NUMAKER_PFM_M487',
    u'1305': u'NUMAKER_PFM_M2351',
    u'1306': u'NUMAKER_PFM_NANO130',
    u'1307': u'NUMAKER_PFM_NUC240',
    u'1549': u'LPC1549',
    u'1600': u'LPC4330_M4',
    u'1605': u'LPC4330_M4',
    u'2000': u'EFM32_G8XX_STK',
    u'2005': u'EFM32HG_STK3400',
    u'2010': u'EFM32WG_STK3800',
    u'2015': u'EFM32GG_STK3700',
    u'2020': u'EFM32LG_STK3600',
    u'2025': u'EFM32TG_STK3300',
    u'2030': u'EFM32ZG_STK3200',
    u'2035': u'EFM32PG_STK3401',
    u'2040': u'EFM32PG12_STK3402',
    u'2041': u'TB_SENSE_12',
    u'2045': u'TB_SENSE_1',
    u'2100': u'XBED_LPC1768',
    u'2201': u'WIZWIKI_W7500',
    u'2202': u'WIZWIKI_W7500ECO',
    u'2203': u'WIZWIKI_W7500P',
    u'2500': u'ADV_WISE_1570',
    u'3001': u'LPC11U24',
    u'4000': u'LPC11U35_Y5_MBUG',
    u'4005': u'NRF51822_Y5_MBUG',
    u'4100': u'MOTE_L152RC',
    u'4337': u'LPC4337',
    u'4500': u'DELTA_DFCM_NNN40',
    u'4501': u'DELTA_DFBM_NQ620',
    u'4502': u'DELTA_DFCM_NNN50',
    u'4600': u'REALTEK_RTL8195AM',
    u'5000': u'ARM_MPS2',
    u'5001': u'ARM_MPS2_M0',
    u'5002': u'ARM_BEETLE_SOC',
    u'5003': u'ARM_MPS2_M0P',
    u'5005': u'ARM_MPS2_M0DS',
    u'5007': u'ARM_MPS2_M1',
    u'5009': u'ARM_MPS2_M3',
    u'5011': u'ARM_MPS2_M4',
    u'5015': u'ARM_MPS2_M7',
    u'5020': u'HOME_GATEWAY_6LOWPAN',
    u'5500': u'RZ_A1H',
    u'5501': u'GR_LYCHEE',
    u'6660': u'NZ32_SC151',
    u'7010': u'BLUENINJA_CDP_TZ01B',
    u'7011': u'TMPM066',
    u'7402': u'MBED_BR_HAT',
    u'7778': u'TEENSY3_1',
    u'8001': u'UNO_91H',
    u'9001': u'LPC1347',
    u'9002': u'LPC11U24',
    u'9003': u'LPC1347',
    u'9004': u'ARCH_PRO',
    u'9006': u'LPC11U24',
    u'9007': u'LPC11U35_501',
    u'9008': u'XADOW_M0',
    u'9009': u'ARCH_BLE',
    u'9010': u'ARCH_GPRS',
    u'9011': u'ARCH_MAX',
    u'9012': u'SEEED_TINY_BLE',
    u'9900': u'NRF51_MICROBIT',
    u'C002': u'VK_RZ_A1H',
    u'C005': u'MTM_MTCONNECT04S',
    u'C006': u'VBLUNO51',
    u'C030': u'UBLOX_C030_U201',
    u'C031': u'UBLOX_C030_N211',
    u'C032': u'UBLOX_C030_R404M',
    u'C033': u'UBLOX_C030_R410M',
    u'C034': u'UBLOX_C030_S200',
    u'C035': u'UBLOX_C030_R3121',
    u'FFFF': u'K20 BOOTLOADER',
    u'RIOT': u'RIOT',
}

class PlatformDatabase(object):
    """Represents a union of multiple platform database files.
    Handles inter-process synchronization of database files.
    """

    target_id_pattern = re.compile(r'^[a-fA-F0-9]{4}$')

    def __init__(self, database_files, primary_database=None):
        """Construct a PlatformDatabase object from a series of platform database files"""
        self._prim_db = primary_database
        if not self._prim_db and len(database_files) == 1:
            self._prim_db = database_files[0]
        self._dbs = OrderedDict()
        self._keys = set()
        for db in database_files:
            try:
                new_db = json.load(open(db, encoding="utf-8"))
                duplicates = set(self._keys).intersection(set(new_db.keys()))
                if duplicates:
                    logger.warning(
                        "Duplicate platform ids found: %s,"
                        " ignoring the definitions from %s",
                        " ".join(duplicates), db)
                self._dbs[db] = new_db
                self._keys = self._keys.union(new_db.keys())
            except IOError as exc:
                if db is LOCAL_PLATFORM_DATABASE:
                    logger.warning(
                        "Could not open local platform database at %s; "
                        "Recereating", db)
                    try:
                        makedirs(dirname(db))
                    except OSError:
                        pass
                    with open(db, "w", encoding="utf-8") as out:
                        out.write(unicode(json.dumps(DEFAULT_PLATFORM_DB)))
                    new_db = copy(DEFAULT_PLATFORM_DB)
                    self._dbs[db] = new_db
                    self._keys = self._keys.union(new_db.keys())
                else:
                    if db is not self._prim_db:
                        logger.error(
                            "Could not open platform database %s: %s; skipping",
                            db, str(exc))
                    self._dbs[db] = {}
            except ValueError as exc:
                logger.error(
                    "Could not parse platform database %s because %s; "
                    "skipping", db, str(exc))
                self._dbs[db] = {}

    def items(self):
        for db in self._dbs.values():
            for entry in db.items():
                yield entry

    def all_ids(self):
        return iter(self._keys)

    def get(self, index, default=None):
        """Standard lookup function. Works exactly like a dict"""
        for db in self._dbs.values():
            maybe_answer = db.get(index, None)
            if maybe_answer:
                return maybe_answer

        return default

    def _update_db(self):
        if self._prim_db:
            lock = InterProcessLock("%s.lock" % self._prim_db)
            acquired = lock.acquire(blocking=False)
            if not acquired:
                logger.debug("Waiting 60 seconds for file lock")
                acquired = lock.acquire(blocking=True, timeout=60)
            if acquired:
                try:
                    with open(self._prim_db, "w", encoding="utf-8") as out:
                        out.write(unicode(
                            json.dumps(self._dbs[self._prim_db])))
                    return True
                finally:
                    lock.release()
            else:
                logger.error("Could not update platform database: "
                             "Lock acquire failed after 60 seconds")
                return False
        else:
            logger.error("Can't update platform database: "
                         "destination database is ambiguous")
            return False

    def add(self, id, platform_name, permanent=False):
        """Add a platform to this database, optionally updating an origin
        database
        """
        if self.target_id_pattern.match(id):
            if self._prim_db:
                self._dbs[self._prim_db][id] = platform_name
            else:
                next(iter(self._dbs.values()))[id] = platform_name
            self._keys.add(id)
            if permanent:
                self._update_db()
        else:
            raise ValueError("Invald target id: %s" % id)

    def remove(self, id, permanent=False):
        """Remove a platform from this database, optionally updating an origin
        database
        """
        logger.debug("Trying remove of %s", id)
        if id is '*':
            self._dbs[self._prim_db] = {}
        for db in self._dbs.values():
            if id in db:
                logger.debug("Removing id...")
                removed = db[id]
                del db[id]
                self._keys.remove(id)
                if permanent:
                    self._update_db()
                return removed



#!/usr/bin/env python
"""
mbed SDK
Copyright (c) 2011-2015 ARM Limited

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import unittest
import os
import errno
import logging
import re
import pkg_resources
import json
from mock import patch, MagicMock
from copy import deepcopy
from six import StringIO

import mbed_lstools.main as cli


try:
    basestring
except NameError:
    # Python 3
    basestring = str

class CLIComands(unittest.TestCase):
    """ Test the CLI
    """

    def setUp(self):
        self._stdout = patch('sys.stdout', new_callable=StringIO)
        self.stdout = self._stdout.start()
        self.mbeds = MagicMock()
        self.args = MagicMock()
        self.mbeds.list_mbeds.return_value = [
            {'platform_name': 'foo', 'platform_name_unique': 'foo[0]',
             'mount_point': 'a mount point', 'serial_port': 'a serial port',
             'target_id': 'DEADBEEF', 'daplink_version': 'v12345'
            }
        ]

    def tearDown(self):
        self._stdout.stop()

    def test_print_version(self):
        cli.print_version(self.mbeds, self.args)
        self.assertIn(pkg_resources.require('mbed-ls')[0].version,
                      self.stdout.getvalue())

    def test_print_table(self):
        cli.print_table(self.mbeds, self.args)
        for d in self.mbeds.list_mbeds.return_value:
            for v in d.values():
                self.assertIn(v, self.stdout.getvalue())

    def test_print_simple(self):
        cli.print_simple(self.mbeds, self.args)
        for d in self.mbeds.list_mbeds.return_value:
            for v in d.values():
                self.assertIn(v, self.stdout.getvalue())

    def test_mbeds_as_json(self):
        cli.mbeds_as_json(self.mbeds, self.args)
        self.assertEqual(self.mbeds.list_mbeds.return_value,
                         json.loads(self.stdout.getvalue()))

    def test_json_by_target_id(self):
        cli.json_by_target_id(self.mbeds, self.args)
        out_dict = json.loads(self.stdout.getvalue())
        for d in out_dict.values():
            self.assertIn(d, self.mbeds.list_mbeds.return_value)

    def test_json_platforms(self):
        cli.json_platforms(self.mbeds, self.args)
        platform_names = [d['platform_name'] for d
                          in self.mbeds.list_mbeds.return_value]
        for name in json.loads(self.stdout.getvalue()):
            self.assertIn(name, platform_names)

    def test_json_platforms_ext(self):
        cli.json_platforms_ext(self.mbeds, self.args)
        platform_names = [d['platform_name'] for d
                          in self.mbeds.list_mbeds.return_value]
        for name in json.loads(self.stdout.getvalue()).keys():
            self.assertIn(name, platform_names)

    def test_list_platform(self):
        self.mbeds.list_manufacture_ids.return_value ="""
        foo
        bar
        baz
        """
        cli.list_platforms(self.mbeds, self.args)
        self.assertIn(self.mbeds.list_manufacture_ids.return_value,
                      self.stdout.getvalue())

class CLIParser(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_parse_cli_defaults(self):
        args = cli.parse_cli([])
        assert callable(args.command)

    def test_parse_cli_conflict(self):
        try:
            args = cli.parse_cli(["-j", "-J"])
            assert False, "Parsing conflicting options should have failed"
        except:
            pass

    def test_parse_cli_single_param(self):
        for p in ['j', 'J', 'p', 'P', '-version', 'd', 'u']:
            args = cli.parse_cli(['-' + p])
            assert callable(args.command)

class CLISetup(unittest.TestCase):
    def test_start_logging(self):
        cli.start_logging()
